// Date Class with operators
//  uses mktime, localtime, strftime for calendar calculations

class Date {
public:
    // Constructor initializes and rationalizes
    Date(int month, int day, int year) {
        if (year >= 1900) year -= 1900;
        if (year >= 0 && year <= 70) year += 100;
        tm.tm_mday = day; tm.tm_mon = month - 1; tm.tm_year = year;
        // Update stime from tm struct
        tm.tm_sec = tm.tm_min = 0; tm.tm_hour = 12;
        stime = mktime(&tm);
    }

    // Operators to add/subtract number of days
    //  This is one way to define increment - as adding 1 - assumes
    //  you have defined what adding an integer does
    Date & operator++() { *this += 1; return *this; }
    Date operator++(int) { Date temp = *this; *this += 1; return temp; }
    Date & operator--() { *this-=1; return *this; }
    Date operator--(int) { Date temp = *this; *this -= 1; return temp; }
    Date & operator+=(int i) { *this=*this+i; return *this;}
    Date & operator-=(int i) { *this=*this-i; return *this;}
    Date operator+(int days) const { return addDaysToDate(days);}
    Date operator-(int days) const { return addDaysToDate(-days); }
    int operator-(const Date &param){
	    return difftime((*this).stime, param.stime)/day_secs;
    }

    // Comparison of Dates
    //  This is one way to define ==, using subtraction defined above
    bool operator==(const Date & rhs) const { cout << *this << " THESE DATES ARE THE SAME! " << rhs << endl; return *this - rhs == 0; }
    bool operator<(const Date & rhs) const { return *this-rhs>0; }
    bool operator>(const Date & rhs) const { return *this-rhs<0; }
    bool operator<=(const Date & rhs) const { return ((*this==rhs)||(*this<rhs)); }
    bool operator>=(const Date & rhs) const { return ((*this==rhs)||(*this>rhs)); }
    bool operator!=(const Date & rhs) const { return !(*this==rhs);}
      // ????   Add some code here - more comparison operators 

    // Conversion to string "Month NN, YYYY"
    operator const char * () const {
        static char buf[100];
        strftime(buf, sizeof(buf), "%B %d, %Y", (&tm));
        return buf;
    }

private:
    struct tm tm;    // Standard time structure
    time_t stime;    // Seconds since the epoch

    static const int day_secs = 60*60*24;  // Seconds per day

    // Private utility method to add days to date
    Date addDaysToDate(int ndays) const {
        if (ndays == 0) return *this;
        Date result = *this; // Copy of self for return
        result.stime += (ndays * day_secs); // Increase time in seconds
        // Update tm struct from stime
        result.tm = *(localtime(&(result.stime)));
        return result;
    }

};

// Output operator
ostream & operator<<(ostream & os, const Date & d) {
    return (os << (const char *)d);
}
